/*
   Module:  bio.c
   Date:    August 1995
   Version: 1.1b
   Author:  Dave Lutz
   Email:   lutz@scp.rochester.edu or lutz@bcs.rochester.edu
   Copyright: 1992, 1995 University of Rochester, Psychology Dept.

   Disclaimer:  This software is distributed free of charge.  As such, it
                comes with ABSOLUTELY NO WARRANTY.  The user of the software
                assumes ALL RISKS associated with its use.

   Purpose: This module provides a set of buffered input/output routines.
            It provides routines needed to open input and output files, and
            to read and write data.

   Functions Provided:

            bopenin
            bopenout
            bclosein
            bcloseout
            bgetstr
            bputarr

   Functions Required:

            none

*/




#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <malloc.h>
#include "bio.h"


/*
   Function: bopenin
   Purpose:  Allocate the input buffer and open the input file for reading.

   Pre: name is a pointer to an ascii string containing the name of the 
        input file.
        bfpp is a pointer to storage for a pointer to a BFILE.

   Post:  The input buffer is allocated.
          The input file is opened for reading.
          If the buffer can't be allocated, *bfpp is set to NULL and
          NOBUFF is returned.
          If the input file can't be opened, *bfpp is set to NULL and
          BADOPEN is returned.
          Otherwise, TRUE is returned.
*/

#ifdef __STDC__
int bopenin (char *name, BFILE **bfpp)
#else
int bopenin (name,bfpp)
   char *name;
   BFILE **bfpp;
#endif
{
   /* first try to allocate a BFILE */
   *bfpp = (BFILE *) malloc (sizeof (BFILE));
   if (*bfpp == (BFILE *)NULL)
      return (NOBUFF);

   /* now try to allocate the buffer */
   (*bfpp)->buf = (char *) malloc ((size_t)(BUFFSIZE*sizeof(char)));  
   if ((*bfpp)->buf == (char *)NULL) {
      free ((void *)*bfpp);
      *bfpp = (BFILE *)NULL;
      return (NOBUFF);
   }

   /* now attempt to open the input file */
   (*bfpp)->fd = open (name,O_RDONLY);
   if ((*bfpp)->fd < 0) {
      free ((void *)(*bfpp)->buf);
      free ((void *)*bfpp);
      *bfpp = (BFILE *)NULL;
      return (BADOPEN);
   }

   /* now set buf_ctr to zero so the first read will cause the buffer to be 
      filled from the input file
   */
   (*bfpp)->buf_ctr = 0;

   (*bfpp)->buf_ptr = (*bfpp)->buf;
   (*bfpp)->eof=FALSE;
   return (TRUE);
}



/*
   Function: bopenout
   Purpose:  Allocate the output buffer, and open the output file for
             writing.

   Pre: name is a pointer to an ascii string containing the name of the
        output file.
        bfpp is a pointer to storage for a pointer to a BFILE.

   Post:  The output buffer is allocated.
          The output file is opened for writing.
          If the output buffer can't be allocated, *bfpp is set to NULL and
          NOBUFF is returned.
          If the output file can't be opened, *bfpp is set to NULL and
          BADOPEN is returned.
          Otherwise, TRUE is returned.
*/

#ifdef __STDC__
int bopenout (char *name, BFILE **bfpp)
#else
int bopenout (name,bfpp)
   char *name;
   BFILE **bfpp;
#endif
{
   /* first try to allocate a BFILE */
   *bfpp = (BFILE *) malloc (sizeof (BFILE));
   if (*bfpp == (BFILE *)NULL)
      return (NOBUFF);

   /* now try to allocate the buffer */
   (*bfpp)->buf = (char *) malloc ((size_t) (BUFFSIZE*sizeof(char)));  
   if ((*bfpp)->buf == (char *)NULL) {
      free ((void *)*bfpp);
      *bfpp = (BFILE *)NULL;
      return (NOBUFF);
   }

   /* now attempt to open the output file */
   (*bfpp)->fd = open (name,O_CREAT|O_TRUNC|O_WRONLY,S_IREAD|S_IWRITE);
   if ((*bfpp)->fd < 0) {
      free ((void *)(*bfpp)->buf);
      free ((void*)*bfpp);
      *bfpp = (BFILE *)NULL;
      return (BADOPEN);
   }
   
   /* now set buf_ptr to point to the beginning of the buffer so the first 
      write will go into the buffer.
   */
   (*bfpp)->buf_ptr = (*bfpp)->buf;

   (*bfpp)->buf_ctr = 0;
   (*bfpp)->eof = FALSE;
   return (TRUE);
}



/*
   Function: bclosein
   Purpose:  Discard the input buffer, close the input file, and deallocate 
             the input buffer.

   Pre:  bfpp is a pointer to a pointer to a BFILE.
         If *bfpp does not point to an open BFILE, it is NULL.

   Post:  If *bfpp = NULL, TRUE is returned.
          Otherwise, the contents of the input buffer are discarded, the 
          input file is closed, and the input buffer is deallocated.
          If an error occurs during the above process, FALSE is returned.
*/

#ifdef __STDC__
int bclosein(BFILE **bfpp)
#else
int bclosein(bfpp)
   BFILE **bfpp;
#endif
{
   if (*bfpp == (BFILE *)NULL)
      return (TRUE);
   free ((void *)(*bfpp)->buf);
   if (close((*bfpp)->fd) < 0) {
       free ((void *)*bfpp);
       return (FALSE);
   }
   free ((void *)*bfpp);
   return (TRUE);
}



/*
   Function: bcloseout
   Purpose:  Flush the output buffer, close the output file, and deallocate 
             the output buffer.

   Pre:  bfpp is a pointer to a pointer to a BFILE.
         If *bfpp does not point to an open BFILE, it is NULL.

   Post:  If *bfpp = NULL, TRUE is returned.
          Otherwise, the output buffer is flushed, the output file is closed, 
          and the output buffer is deallocated.
          If an error occurs during the above process, FALSE is returned.
*/

#ifdef __STDC__
int bcloseout(BFILE **bfpp)
#else
int bcloseout(bfpp)
   BFILE **bfpp;
#endif
{
   if (*bfpp == (BFILE *)NULL)
      return (TRUE);
   if (write((*bfpp)->fd,(*bfpp)->buf,(unsigned)(*bfpp)->buf_ctr) != 
       (*bfpp)->buf_ctr) {
      free ((void *)(*bfpp)->buf);
      (void) close ((*bfpp)->fd);
      free ((void *)*bfpp);
      return (FALSE);
   }
   free ((void *)(*bfpp)->buf);
   if (close((*bfpp)->fd) < 0) {
       free ((void *)*bfpp);
       return (FALSE);
   }
   free ((void *)*bfpp);
   return (TRUE);
}



/*
   Function: bgetstr
   Purpose:  Read a string from the input buffer, possibly refilling the
             buffer from the input file.  Data is read up to the next
             line feed character.

   Pre:  bfp is a pointer to a BFILE that has been opened for reading.
         max is the maximum number of chars to be stored in string.
         string is a pointer to storage for the input string.
         There is enough storage available in string to hold "max"+1 chars.

   Post: An attempt is made to read a string from the input buffer and store
         it in "string".
         Reading stops at eof or the first line feed.  The line feed,
         although included in the byte count, is replaced with a NULL.
         If the input buffer is emptied during the process, the input file is 
         read to refill the buffer.
         If "max" chars are read without encountering a line feed, the extra
         chars are discarded.
         The number of bytes read from the input buffer is returned.
         If end_of_file has already been reached, 0 is returned.
         If an error occurs, -1 is returned.
*/

#ifdef __STDC__
int bgetstr (BFILE *bfp, int max, char *string)
#else
int bgetstr (bfp, max, string)
   BFILE *bfp;
   int max;
   char *string;
#endif
{
   int cnt=0, done=FALSE;

   if (bfp->eof == TRUE)
      return (0);
   if (bfp->buf_ptr >= (bfp->buf + bfp->buf_ctr)) {
      /* the buffer is empty, refill it from the input file */
      switch (bfp->buf_ctr = read(bfp->fd,bfp->buf,
                                    (unsigned)BUFFSIZE)) {
         case 0:
            bfp->eof = TRUE;
            return (0);
         case -1:
            return (-1);
         default:
            bfp->buf_ptr = bfp->buf;
            /* bfp->buf_ctr was set by the read funtion */
            break;
      }
   }
   while (!done && cnt < max) {
      if (bfp->buf_ptr >= (bfp->buf + bfp->buf_ctr)) {
         /* the buffer is empty, refill it from the input file */
         switch (bfp->buf_ctr = read(bfp->fd,bfp->buf,
                                       (unsigned)BUFFSIZE)) {
            case 0:
               done = bfp->eof = TRUE;
               break;
            case -1:
               return (-1);
            default:
               bfp->buf_ptr = bfp->buf;
               /* bfp->buf_ctr was set by the read funtion */
               break;
         }
      }
      if (!bfp->eof) {
         if ((*string = *(bfp->buf_ptr)) == '\n') {
            *string = (char)NULL;
            cnt++;
            bfp->buf_ptr++;
            done = TRUE;
         } else {
            string++;
            cnt++;
            bfp->buf_ptr++;
         }
      }
   }
   if (!done) {
      /* we have read max characters without finding a line feed.  Read until
         line feed or eof, but don't store in string.
      */
      while (!done) {
         if (bfp->buf_ptr >= (bfp->buf + bfp->buf_ctr)) {
            /* the buffer is empty, refill it from the input file */
            switch (bfp->buf_ctr = read(bfp->fd,bfp->buf,
                                          (unsigned)BUFFSIZE)) {
               case 0:
                  done = bfp->eof = TRUE;
                  break;
               case -1:
                  return (-1);
               default:
                  bfp->buf_ptr = bfp->buf;
                  /* bfp->buf_ctr was set by the read funtion */
                  break;
           }
         }
         if (!bfp->eof) {
            if (*(bfp->buf_ptr) == '\n')
               done = TRUE;
            cnt++;
            bfp->buf_ptr++;
         }
      }
   }
   return (cnt);
}



/*
   Function: bputarr
   Purpose:  Write a char array to the output buffer, possibly causing the 
             buffer to be written to the output file.

   Pre: bfp is a pointer to a BFILE that has been opened for writing.
        num is the number of chars to be written to the output buffer.
        chrarr is a pointer to the char array to be written.

   Post: An attempt is made to copy num chars from chrarr to the output 
         buffer.
         If the buffer becomes full during the process, an attempt is made
         to write the buffer to the output file.
         If an error occurs, FALSE is returned, otherwise TRUE is returned.
*/

#ifdef __STDC__
int bputarr (BFILE *bfp, int num, char *chrarr)
#else
int bputarr (bfp, num, chrarr)
   BFILE *bfp;
   int num;
   char *chrarr;
#endif
{
   int cnt=0;

   if (bfp->buf_ptr >= bfp->buf+BUFFSIZE) {
      if (write(bfp->fd,bfp->buf,(unsigned)bfp->buf_ctr) != bfp->buf_ctr)
         return (FALSE);
      bfp->buf_ptr = bfp->buf;
      bfp->buf_ctr = 0;
   }
   while (cnt < num) {
      if (bfp->buf_ptr >= bfp->buf+BUFFSIZE) {
         if (write(bfp->fd,bfp->buf,(unsigned)bfp->buf_ctr) != bfp->buf_ctr)
            return (FALSE);
         bfp->buf_ptr = bfp->buf;
         bfp->buf_ctr = 0;
      }
      *(bfp->buf_ptr++) = *chrarr++;
      cnt++;
      bfp->buf_ctr++;
   }
   return (TRUE);
}
