/*
   Module:  bio.h
   Date:    August 1995
   Version: 1.1b
   Author:  Dave Lutz
   Email:   lutz@scp.rochester.edu or lutz@bcs.rochester.edu
   Copyright: 1992, 1995 University of Rochester, Psychology Dept.

   Disclaimer:  This software is distributed free of charge.  As such, it
                comes with ABSOLUTELY NO WARRANTY.  The user of the software
                assumes ALL RISKS associated with its use.

   Purpose: This module provides the function prototypes for a set of buffered
            input/output routines.  It provides routines needed to open input
            and output files, and to read and write data.

            A datatype (BFILE) is also provided by this module.  This
            datatype is the basic structure used by all of the functions in
            this module.

            Two special return codes are defined here as well.  NOBUFF
            indicates a failure to allocate storage for a BFILE buffer, and
            BADOPEN indicates a problem opening a file.

   Functions Provided:

            bopenin
            bopenout
            bclosein
            bcloseout
            bgetstr
            bputarr

*/


  /* BUFFSIZE has been set to get the maximum number of 2048 byte clusters
     and still be less than 32767
  */
#define BUFFSIZE 30720
#define BFILE struct _bfile

#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif
#ifndef NULL
#define NULL 0L
#endif

#define NOBUFF  11
#define BADOPEN 12


BFILE {
   char *buf,     /* start of file buffer */
        *buf_ptr; /* current location in buffer */
   int  fd,       /* file descriptor for the file */
        buf_ctr,  /* number of bytes currently in buffer */
        eof;      /* BOOLEAN flag, only used on input files */
};



/*
   Function: bopenin
   Purpose:  Allocate the input buffer and open the input file for reading.

   Pre: name is a pointer to an ascii string containing the name of the 
        input file.
        bfpp is a pointer to storage for a pointer to a BFILE.

   Post:  The input buffer is allocated.
          The input file is opened for reading.
          If the buffer can't be allocated, *bfpp is set to NULL and
          NOBUFF is returned.
          If the input file can't be opened, *bfpp is set to NULL and
          BADOPEN is returned.
          Otherwise, TRUE is returned.
*/

#ifdef __STDC__
extern int bopenin (char *name, BFILE **bfpp);
#else
extern int bopenin ();
#endif



/*
   Function: bopenout
   Purpose:  Allocate the output buffer, and open the output file for
             writing.

   Pre: name is a pointer to an ascii string containing the name of the
        output file.
        bfpp is a pointer to storage for a pointer to a BFILE.

   Post:  The output buffer is allocated.
          The output file is opened for writing.
          If the output buffer can't be allocated, *bfpp is set to NULL and
          NOBUFF is returned.
          If the output file can't be opened, *bfpp is set to NULL and
          BADOPEN is returned.
          Otherwise, TRUE is returned.
*/

#ifdef __STDC__
extern int bopenout (char *name, BFILE **bfpp);
#else
extern int bopenout ();
#endif



/*
   Function: bclosein
   Purpose:  Discard the input buffer, close the input file, and deallocate 
             the input buffer.

   Pre:  bfpp is a pointer to a pointer to a BFILE.
         If *bfpp does not point to an open BFILE, it is NULL.

   Post:  If *bfpp = NULL, TRUE is returned.
          Otherwise, the contents of the input buffer are discarded, the 
          input file is closed, and the input buffer is deallocated.
          If an error occurs during the above process, FALSE is returned.
*/

#ifdef __STDC__
extern int bclosein(BFILE **bfpp);
#else
extern int bclosein();
#endif



/*
   Function: bcloseout
   Purpose:  Flush the output buffer, close the output file, and deallocate 
             the output buffer.

   Pre:  bfpp is a pointer to a pointer to a BFILE.
         If *bfpp does not point to an open BFILE, it is NULL.

   Post:  If *bfpp = NULL, TRUE is returned.
          Otherwise, the output buffer is flushed, the output file is closed, 
          and the output buffer is deallocated.
          If an error occurs during the above process, FALSE is returned.
*/

#ifdef __STDC__
extern int bcloseout(BFILE **bfpp);
#else
extern int bcloseout();
#endif



/*
   Function: bgetstr
   Purpose:  Read a string from the input buffer, possibly refilling the
             buffer from the input file.  Data is read up to the next
             line feed character.

   Pre:  bfp is a pointer to a BFILE that has been opened for reading.
         max is the maximum number of chars to be stored in string.
         string is a pointer to storage for the input string.
         There is enough storage available in string to hold "max"+1 chars.

   Post: An attempt is made to read a string from the input buffer and store
         it in "string".
         Reading stops at eof or the first line feed.  The line feed,
         although included in the byte count, is replaced with a NULL.
         If the input buffer is emptied during the process, the input file is 
         read to refill the buffer.
         If "max" chars are read without encountering a line feed, the extra
         chars are discarded.
         The number of bytes read from the input buffer is returned.
         If end_of_file has already been reached, 0 is returned.
         If an error occurs, -1 is returned.
*/

#ifdef __STDC__
extern int bgetstr (BFILE *bfp, int max, char *string);
#else
extern int bgetstr ();
#endif



/*
   Function: bputarr
   Purpose:  Write a char array to the output buffer, possibly causing the 
             buffer to be written to the output file.

   Pre: bfp is a pointer to a BFILE that has been opened for writing.
        num is the number of chars to be written to the output buffer.
        chrarr is a pointer to the char array to be written.

   Post: An attempt is made to copy num chars from chrarr to the output 
         buffer.
         If the buffer becomes full during the process, an attempt is made
         to write the buffer to the output file.
         If an error occurs, FALSE is returned, otherwise TRUE is returned.
*/

#ifdef __STDC__
extern int bputarr (BFILE *bfp, int num, char *chrarr);
#else
extern int bputarr ();
#endif
