/*
   Module: odm.c
   Version: 1.1 Beta
   Original Author: Roger N. Clark
   Translated to C by: Dave Lutz
   Reference: "Visual Astronomy of the Deep Sky", Clark R., 1990 Sky Publishing
              Corporation, ISBN  0-933-346-54-9, and Cambridge University
              Press, ISBN 0-521-36155-9
   Copyright: The original work is Copyright Roger N. Clark 1990, and is used
              by permission.
              This version is Copyright David L. Lutz 1995.

   Disclaimer:  This software is distributed free of charge.  As such, it
                comes with ABSOLUTELY NO WARRANTY.  The user of the software
                assumes ALL RISKS associated with its use.

   Purpose: This module contains all of the calculations used to determine
            the Optimum Detection Magnification, C1 log contrast, and log
            threshold contrast for an object, given the aperture of a telescope
            and the background surface brightness of the sky.

   Notes: This code was taken from "Visual Astronomy of the Deep Sky" (see
          reference above).  The original code was written in Fortran, and was
          translated to C for better portability.
*/

#ifdef __STDC__
double thrcon(double angle, double sb);
double opvang(double sb);
/* linked in from system libraries */
double log10(double x);
double pow(double x, double y);
#else
double thrcon();
double opvang();
/* linked in from system libraries */
double log10();
double pow();
#endif

/*
   Function: calcenvdat
   Purpose: Calculate environmental data

        tele  = telescope's aperture in inches

        tmag  = telescope's limiting magnitude (ie faintest star)
        ipmin = telescope's minimum useful magnification
*/

#ifdef __STDC__
void calcenvdat(double tele, double *tmag, int *ipmin)
#else
void calcenvdat(tele,tmag,ipmin)
   double tele, *tmag;
   int   *ipmin;
#endif
{
   double dmm;  /* telescope's aperture in millimeters */

   /* determine the faintest star based on equation 4.1 (= tmag):
      dsube = aperture of eye in millimeters
      trans = telescope transmission factor
         trans = 0.7
         dmm = tele*25.4
         dsube = 7.5
         tmag = 8.5 + 2.5 * log10(dmm*dmm*trans/(dsube*dsube))
         which reduces to equation 4.2:
   */
   dmm = tele * 25.4;
   *tmag = 3.7 + 2.5 * log10(dmm * dmm);

   /* minimum useful magnification for telescope */
   *ipmin = (int) (tele * 3.375 + 0.5);
}


/*
   Function: calcobjdat
   Purpose: calculate object data

        xmag = total visual magnitude of object
        size1 = one dimension of the object (in arc-minutes)
        size2 = the other dimension of the object (in arc-minutes)
        sb0  = background surface brightness

        sb = surface brightness
        c1 = C1 contrast index
        xminsz = smaller of size1 and size2
        sbs = combined sky/object surface brightness
*/

#ifdef __STDC__
void calcobjdat(double xmag, double size1, double size2, double sb0,
                double *sb, double *c1, double *xminsz, double *sbs)
#else
void calcobjdat(xmag,size1,size2,sb0,sb,c1,xminsz,sbs)
   double xmag, size1, size2, sb0, *sb, *c1, *xminsz, *sbs;
#endif
{
   /* compute the surface brightness (see Appendix E) */
   *sb = xmag + 2.5 * log10 (2827.0 * size1 * size2);

   /* compute the C1 contrast index (See Appendix E) */
   *c1 = -0.4 * (*sb - sb0);

   /* find minimum size */
   *xminsz = (size1 < size2) ? size1 : size2;

   /* add sky to object surface brightness */
   *sbs = -2.5 * log10(pow(10.0,(-0.4 * *sb)) + pow(10.0,(-0.4 * sb0)));
}


/*
   Function: calcodm
   Purpose: calculate the Optimum Detection Magnification, and the related
            surface brightness and contrast values.

        sb0  = background surface brightness
        xminsz = smaller of size1 and size2
        tele = telescope's aperture in inches
        ipmin = telescope's minimum useful magnification
        sbs = combined sky/object surface brightness

        mb = surface brightness reduction at power
        sbp = surface brightness in telescope at magnification= power
        sbm = surface brightness combined with sb reduction
        odm = optimum detection magnification
        xc = contrast of object at odm
*/

#ifdef __STDC__
void calcodm(double sb0, double xminsz, double tele, int ipmin, double sbs,
             double *mb, double *sbp, double *sbm, int *odm, double *xc)
#else
void calcodm(sb0,xminsz,tele,ipmin,sbs,mb,sbp,sbm,odm,xc)
   double sb0, xminsz, tele, sbs, *mb, *sbp, *sbm, *xc;
   int ipmin, *odm;
#endif
{
   double power,   /* current optimum magnifiction guess */
          opower,  /* new optimum magnifiction guess */
          optsiz,  /* optimal size (angle) of object */
          angle;   /* magnified size (angle) of object */

      /* find Optimum Visual Angle : optsiz = opvang(sbm)
         then minimum detection magnification is mdm = optsiz/xminsz
      */

      /* start with initial guess of 100x */
      opower = 100;


      /* find surface brightness reduction in the telescope */
      do {
         power = opower;
         *mb = 5.0 * log10(power/(2.833487 * tele));
         *sbm = sb0 + *mb;
         optsiz = opvang(*sbm);
         opower = optsiz/xminsz;
#ifdef DEBUG1
         printf ("calcodm:\n");
         printf ("D= %4.0f inches, guess=%6.1f, new guess=%6.1f, sbm=%5.1f, optsiz=%6.1f\n\n",tele,power,opower,*sbm,optsiz);
#endif

         if (opower < ipmin)
            opower = ipmin;

      } while (((opower - power) > 0.1) || ((opower - power) < -0.1));
      /* we've converged on the ODM, which is now in opower */

      *odm = (int) (opower + 0.5);

      /* check if odm < min useful power, and limit max power */
      if (*odm < ipmin)
         *odm = ipmin;
      if (*odm > 9999)
         *odm = 9999;

      /* check if object is visible in each telescope, and determine if it has
         a high enough contrast to be detectable
      */
      angle = *odm * xminsz;
      *xc = log10(thrcon(angle,*sbm));
      /* sbp = surface brightness in telescope at magnification= power */
      *sbp = sbs + *mb;
}


/*
   Function: thrcon
   Purpose: Compute the contrast threshold for a given angle by two dimensional
            interpolation.

        angle = angle in arc-minutes
        sb = surface brightness
        angx = log10 angles (arc-min) data for 7 values of contrast
        sbx = log10 contrast for angx values for each surface brightness
              ranging from 4 to 27 mag/arc-sec

        angx and sbx are used to interpolate to the thrcon given by ang.
*/

#ifdef __STDC__
double thrcon(double angle, double sb)
#else
double thrcon(angle,sb)
   double angle, sb;
#endif
{

static double angx[7] = {
   -0.2255, 0.5563, 0.9859, 1.2601, 1.7419, 2.0828, 2.5563
};

static double sbx0[7] = {
   /* mag/arc-sec = 4 */
   -0.376911, -1.806444, -2.336825, -2.460093, -2.546943, -2.560968, -2.565971
};

static double sbx1[7] = {
   /* mag/arc-sec = 5 */
   -0.331451, -1.774717, -2.333688, -2.460781, -2.546521, -2.560729, -2.565756
};

static double sbx2[7] = {
   /* mag/arc-sec = 6 */
   -0.268183, -1.734482, -2.331011, -2.460521, -2.546681, -2.560819, -2.565837
};

static double sbx3[7] = {
   /* mag/arc-sec = 7 */
   -0.198177, -1.685101, -2.313951, -2.457228, -2.548051, -2.561529, -2.566460
};

static double sbx4[7] = {
   /* mag/arc-sec = 8 */
   -0.123815, -1.625192, -2.279119, -2.446245, -2.546321, -2.559690, -2.564570
};

static double sbx5[7] = {
   /* mag/arc-sec = 9 */
   -0.042444, -1.552937, -2.229734, -2.421422, -2.534318, -2.550130, -2.555202
};

static double sbx6[7] = {
   /* mag/arc-se6 = 10 */
    0.049842, -1.465538, -2.165945, -2.376266, -2.504717, -2.526875, -2.533309
};

static double sbx7[7] = {
   /* mag/arc-sec = 11 */
    0.159558, -1.358054, -2.081004, -2.303572, -2.449928, -2.482261, -2.493710
};

static double sbx8[7] = {
   /* mag/arc-sec = 12 */
    0.293354, -1.225640, -1.967364, -2.196521, -2.363052, -2.409170, -2.431807
};

static double sbx9[7] = {
   /* mag/arc-sec = 13 */
    0.455739, -1.067267, -1.818647, -2.053094, -2.244490, -2.308296, -2.349087
};

static double sbx10[7] = {
   /* mag/arc-sec = 14 */
    0.649990, -0.884068, -1.629165, -1.874059, -2.098851, -2.184839, -2.250534
};

static double sbx11[7] = {
   /* mag/arc-sec = 15 */
    0.880777, -0.668706, -1.396670, -1.661059, -1.928370, -2.041067, -2.137519
};

static double sbx12[7] = {
   /* mag/arc-sec = 16 */
    1.155840, -0.395243, -1.126426, -1.417576, -1.729969, -1.872722, -2.003406
};

static double sbx13[7] = {
   /* mag/arc-sec = 17 */
    1.482228, -0.041944, -0.824314, -1.147481, -1.502126, -1.676828, -1.842016
};

static double sbx14[7] = {
   /* mag/arc-sec = 18 */
    1.855923,  0.345791, -0.492425, -0.856105, -1.266128, -1.472054, -1.662449
};

static double sbx15[7] = {
   /* mag/arc-sec = 19 */
    2.266949,  0.696000, -0.131546, -0.550963, -1.056193, -1.289210, -1.482659
};

static double sbx16[7] = {
   /* mag/arc-sec = 20 */
    2.676000,  1.088000,  0.206000, -0.321000, -0.880000, -1.137000, -1.362000
};

static double sbx17[7] = {
   /* mag/arc-sec = 21 */
    2.776553,  1.206474,  0.346695, -0.137740, -0.736142, -0.996400, -1.243900
};

static double sbx18[7] = {
   /* mag/arc-sec = 22 */
    2.930431,  1.382138,  0.535251,  0.032813, -0.560534, -0.860600, -1.118700
};

static double sbx19[7] = {
   /* mag/arc-sec = 23 */
    3.163406,  1.610655,  0.770789,  0.253144, -0.389500, -0.703000, -0.968133
};

static double sbx20[7] = {
   /* mag/arc-sec = 24 */
    3.464285,  1.903381,  1.033812,  0.494252, -0.203342, -0.525901, -0.828800
};

static double sbx21[7] = {
   /* mag/arc-sec = 25 */
    3.821092,  2.256431,  1.326535,  0.760529,  0.017200, -0.299200, -0.639400
};

static double sbx22[7] = {
   /* mag/arc-sec = 26 */
    4.221000,  2.632000,  1.699000,  1.132000,  0.286000, -0.051000, -0.408000
};

static double sbx23[7] = {
   /* mag/arc-sec = 27 */
    4.610000,  3.066000,  2.132000,  1.585000,  0.652000,  0.241000, -0.121000
};


static double *sbx[24] = {
   sbx0, sbx1, sbx2, sbx3, sbx4, sbx5, sbx6, sbx7,
   sbx8, sbx9, sbx10, sbx11, sbx12, sbx13, sbx14, sbx15,
   sbx16, sbx17, sbx18, sbx19, sbx20, sbx21, sbx22, sbx23
};

#ifdef DEBUG1
   int          iia;    /* loop counter plus 1 */
#endif
   int          i,      /* loop counter */
                isb1,   /* index into sbx */
                isb2;   /* index into sbx */
   double       ang,    /* log10 of angle */
                x1,     /* contrast interpolated in the angle direction at the
                           first bounding sb value
                        */
                x2,     /* contrast interpolated in the angle direction at the
                           second bounding sb value
                        */
                x;      /* thrcon = 10**x */

   /* find bounding sb indices */
   isb1 = (int)(sb) - 4;
   if (isb1 < 0)
      isb1 = 0;
   if (isb1 > 22)
      isb1 = 22;
   isb2 = isb1 + 1;

   i = 6;
   ang = log10(angle);
   if (ang <= angx[0]) {
      /* ang is less than beginning of table, so extrapolate */
      x1 = sbx[isb1][0];
      x2 = sbx[isb2][0];
   } else if (ang >= angx[6]) {
      /* ang is greater than last in table, so extrapolate */
      x1 = (ang-angx[5]) / (angx[6]-angx[5]) *
           (sbx[isb1][6]-sbx[isb1][5]) + sbx[isb1][5];
      x2 = (ang-angx[5]) / (angx[6]-angx[5]) *
           (sbx[isb2][6]-sbx[isb2][5]) + sbx[isb2][5];
   } else {
      /* find angx values bounding ang */
      for (i=0; (i<=5); i++) {
         if ((angx[i] < ang) && (angx[i+1] >= ang))
            break;
      }
      /* interpolate along the angle direction.
         x1 is the contrast interpolated in the angle direction at the
            first bounding sb value.
         x2 is the contrast interpolated in the angle direction at the
            second bounding sb value.
      */
      x1 = (ang-angx[i]) / (angx[i+1]-angx[i]) *
           (sbx[isb1][i+1]-sbx[isb1][i]) + sbx[isb1][i];
      x2 = (ang-angx[i]) / (angx[i+1]-angx[i]) *
           (sbx[isb2][i+1]-sbx[isb2][i]) + sbx[isb2][i];
   }

   /* now interpolate (or extrapolate) along the sb direction
      compute thrcon from x1, x2
   */
   if (sb < 4.0)
      x = x1;
   else if (sb >= 27.0)
      x = (sb-27.0)*(x2-x1) + x2;
   else
      x = (sb-(double)(int)sb) * (x2-x1) + x1;

   if (x > 37.0)
      x = 37.0;
   else if (x < -37.0)
      x = -37.0;

#ifdef DEBUG1
   printf ("thrcon:\n");
   printf ("i=%d\tisb1=%d\n",i,isb1);
   iia = i + 1;
   if (iia > 6)
      iia = 6;
   printf ("   ang    angx[i] angx[i+1] sbx[isb1][i] sbx[isb1][i+1]\n");
   printf ("%7.3f  %7.3f  %7.3f     %7.3f      %7.3f\n",ang,angx[i],
            angx[iia],sbx[isb1][i],sbx[isb1][iia]);
#endif

   return (pow(10.0,x));
}


/*
   Function: opvang
   Purpose: Compute the optimum visual angle for an object, given the
            background surface brightness.

      sbx = surface brightness data for 18 values of optimum visual angle
      opv = log10 optimum visual angles for sbx values

      sbx and opv are used to interpolate to the opvang of a given sb.
*/

#ifdef __STDC__
double opvang (double sb)
#else
double opvang (sb)
   double sb;
#endif
{

static double sbx[18] = {
   4.0, 9.0, 11.0, 13.0, 14.0, 15.0, 16.0, 17.0, 18.0, 19.0, 20.0, 21.0, 22.0,
   23.0, 24.0, 25.0, 26.0, 27.0
};

static double opv[18] = {
   0.97, 1.01, 1.04, 1.07, 1.11, 1.14, 1.20, 1.26, 1.36, 1.49, 1.62, 1.71,
   1.79, 1.83, 1.86, 1.90, 1.96, 2.07
};

   int          i;      /* loop counter */
   double       x,      /* opvang = 10**x */
                opvang; /* optimum visual angle */

   if (sb <= sbx[0]) {
      /* sb less than beginning of table */
      x = opv[1];
   } else if (sb >= sbx[17]) {
      /* sb greater than last in table */
      x = (sb - sbx[16]) / (sbx[17]-sbx[16]) * (opv[17]-opv[16]) + opv[16];
   } else {
      /* find sbx values bounding sb */
      for (i=0; (i<=16); i++)
         if ((sbx[i] < sb) && (sbx[i+1] >= sb))
            break;
      /* interpolate */
      x = (sb-sbx[i]) / (sbx[i+1]-sbx[i]) * (opv[i+1] - opv[i]) + opv[i];
   }

   /* compute opvang from x */
   if (x > 37.0)
      x = 37.0;
   if (x < -37.0)
      x = -37.0;

#ifdef DEBUG2
   printf ("opvang:\n");
   printf ("i=%d\n",i);
   printf ("   sb     sbx[i] sbx[i+1] opv[i] opv[i+1]\n");
   printf ("%7.3f  %7.3f %7.3f %7.3f %7.3f\n",sb,sbx[i],
            sbx[i+1],opv[i],opv[i+1]);
#endif

   opvang = pow(10.0,x);
   return ((opvang > 360.0) ? 360.0 : opvang);
}
