/*
   Module: odm.h
   Original Author: Roger N. Clark
   Translated to C by: Dave Lutz
   Reference: "Visual Astronomy of the Deep Sky", Clark R., 1990 Sky Publishing
              Corporation, ISBN  0-933-346-54-9, and Cambridge University
              Press, ISBN 0-521-36155-9

   Copyright: The original work is Copyright Roger N. Clark 1990, and is used
              by permission.
              This version is Copyright David L. Lutz 1995.

   Disclaimer:  This software is distributed free of charge.  As such, it
                comes with ABSOLUTELY NO WARRANTY.  The user of the software
                assumes ALL RISKS associated with its use.

   Purpose: This header file should be #included into any program that uses
            the functions defined in odm.c.
*/

#ifndef _odm_h_
#define _odm_h_

/*
   Function: calcenvdat
   Purpose: Calculate environmental data

        tele  = telescope's aperture in inches

        tmag  = telescope's limiting magnitude (ie faintest star)
        ipmin = telescope's minimum useful magnification
*/

#ifdef __STDC__
extern void calcenvdat(double tele, double *tmag, int *ipmin);
#else
extern void calcenvdat();
#endif

/*
   Function: calcobjdat
   Purpose: calculate object data

        xmag = total visual magnitude of object
        size1 = one dimension of the object (in arc-minutes)
        size2 = the other dimension of the object (in arc-minutes)
        sb0  = background surface brightness

        sb = surface brightness
        c1 = C1 contrast index
        xminsz = smaller of size1 and size2
        sbs = combined sky/object surface brightness
*/

#ifdef __STDC__
extern void calcobjdat(double xmag, double size1, double size2, double sb0,
                       double *sb, double *c1, double *xminsz, double *sbs);
#else
extern void calcobjdat();
#endif


/*
   Function: calcodm
   Purpose: calculate the Optimum Detection Magnification, and the related
            surface brightness and contrast values.

        sb0  = background surface brightness
        xminsz = smaller of size1 and size2
        tele = telescope's aperture in inches
        ipmin = telescope's minimum useful magnification
        sbs = combined sky/object surface brightness

        mb = surface brightness reduction at power
        sbp = surface brightness in telescope at magnification= power
        sbm = surface brightness combined with sb reduction
        odm = optimum detection magnification
        xc = contrast of object at odm
*/

#ifdef __STDC__
extern void calcodm(double sb0, double xminsz, double tele, int ipmin,
                    double sbs, double *mb, double *sbp, double *sbm,
                    int *odm, double *xc);
#else
extern void calcodm();
#endif


/*
   Function: thrcon
   Purpose: Compute the contrast threshold for a given angle by two dimensional
            interpolation.

        angle = angle in arc-minutes
        sb = surface brightness
        angx = log10 angles (arc-min) data for 7 values of contrast
        sbx = log10 contrast for angx values for each surface brightness
              ranging from 4 to 27 mag/arc-sec

        angx and sbx are used to interpolate to the thrcon given by ang.
*/

#ifdef __STDC__
extern double thrcon(double angle, double sb);
#else
extern double thrcon();
#endif


/*
   Function: opvang
   Purpose: Compute the optimum visual angle for an object, given the
            background surface brightness.

      sbx = surface brightness data for 18 values of optimum visual angle
      opv = log10 optimum visual angles for sbx values

      sbx and opv are used to interpolate to the opvang of a given sb.
*/

#ifdef __STDC__
extern double opvang (double sb);
#else
extern double opvang();
#endif

#endif
