/*
   Program: sacodm.c
   Version: 1.1 Beta
   Author: David Lutz
   Bug reports to: lutz@scp.rochester.edu or lutz@bcs.rochester.edu

   Notes: This program is based heavily on the teleodm program listed in
	  appendix F of "Visual Astronomy of the Deep Sky", Roger N. Clark,
	  1990, Sky Publishing Corporation, ISBN 0-933-346-54-9, and Cambridge
	  University Press, ISBN 0-521-36155-9. All of the calculations, and
	  the thrcon and opvang functions, are taken from that program. The
	  author and publisher have both generously agreed to allow this
	  program to be released free of charge.

   Copyright: The original work is Copyright Roger N. Clark 1990, and is used
              by permission.
              This version is Copyright David L. Lutz 1995.

   Disclaimer:  This software is distributed free of charge.  As such, it
                comes with ABSOLUTELY NO WARRANTY.  The user of the software
                assumes ALL RISKS associated with its use.

   Purpose:

        This program works with raw data files from the SAC version 6.0 Deep
        Sky Database.  When provided with the aperture of a telescope and the
        background surface brightness of the sky, the program determines 
        the Optimum Detection Magnification, the log contrast at odm, the
        contrast threshold at odm, and the visibility (y/n) for each record.

        There are several reporting options, including a formatted report, a
        raw dump of the input records with the calculated vlues appended as new
        fields, and a pruned raw dump of the input records.  The pruned dump
        includes only those objects which are known to be visible in the given
        telescope, and those objects that do not include enough information to
        make a determination.  It is also possible to produce pruned versions
        of both reports that include the calculated values.

*/

#include <stdio.h>
#include <math.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include "odm.h"
#include "bio.h"

#define FALSE 0
#define TRUE 1
#define BADDAT 2

#define MAXLINE 1024

#define OBJECTSZ 17
#define OTHERSZ 18
#define TYPESZ 5
#define CONSZ 3
#define RASZ 7
#define DECSZ 6
#define MAGSZ 4
#define SUBRSZ 4
#define U2000SZ 3
#define S2000SZ 2
#define DESCRSZ 50
#define SIZESZ 11
#define CLASSSZ 10
#define NSTSSZ 4
#define BRSTRSZ 5
#define NOTESZ 70

struct objrec {
   char rawdat[MAXLINE+1],      /* the raw input line */
        object[OBJECTSZ+1],     /* primary name of object */
        other[OTHERSZ+1],       /* secondary name of object */
        type[TYPESZ+1],         /* object type */
        con[CONSZ+1],           /* constellation */
        ra[RASZ+1],             /* right ascension */
        dec[DECSZ+1],           /* declination */
        mag[MAGSZ+1],           /* magnitude */
        subr[SUBRSZ+1],         /* surface brightness */
        u2000[U2000SZ+1],       /* Uranometria 2000.0 chart number */
        s2000[S2000SZ+1],       /* Sky Atlas 2000.0 chart number */
        descr[DESCRSZ+1],       /* visual description */
        size[SIZESZ+1],         /* size in arc minutes (') or seconds ('') */
        class[CLASSSZ+1],       /* professional classification */
        nsts[NSTSSZ+1],         /* num stars within a cluster */
        brstr[BRSTRSZ+1],       /* magnitude of brightest star in cluster or
                                   central star of planetary nebula
                                */
        note[NOTESZ+1];         /* notes about the object */
};

#ifdef __STDC__
void usage(void);
void getinfile(char *infilename, BFILE **fpp);
void getoutfile(char *outfilename, BFILE **fpp);
int getline(char buff[], int max);
void getenvdat(double *tele, double *sb0);
char *getfield(char *string, int max, char *field);
int getobjdat(BFILE *fp, struct objrec *obj);
int convobjdat(struct objrec *obj, double *xmag, double *size1, double *size2);
void formatout(BFILE *outfp, struct objrec *obj, int odm, double c1, double xc,
               double tele, int ipmin, double tmag, double sb0);
void fulldump(BFILE *outfp, struct objrec *obj, int odm, double c1, double xc);
#else
void usage();
void getinfile();
void getoutfile();
int getline();
void getenvdat();
char *getfield();
int getobjdat();
int convobjdat();
void formatout();
void fulldump();
#endif

#ifdef __STDC__
int main (int argc, char *argv[])
#else
int main (argc, argv)
   int argc;
   char *argv[];
#endif
{

   int          i,      /* loop counter */
                odm,    /* optimum detection magnification */
                ipmin,  /* minimum useful magnification for telescope */
                format, /* operate in format mode? */
                dump,   /* operate in dump mode? */
                prune,  /* operate in prune mode? */
                ret;    /* returned value from function call */
   double       tele,   /* telescope's aperture in inches */
                mb,     /* surface brightness reduction at power */
                sbm,    /* surface brightness combined with sb reduction */
                tmag,   /* telescope's limiting magnitude */
                xmag,   /* total magnitude of object */
                sb0,    /* background surface brightness */
                sb,     /* computed surface brightness of object */
                sbs,    /* sb0 combined with sb */
                sbp,    /* surface brightness of object at odm */
                c1,     /* computed contrast index */
                xc,     /* contrast of object at odm */
                size1,  /* one dimension of the object (in arc-minutes) */
                size2,  /* one dimension of the object (in arc-minutes) */
                xminsz; /* minimum size of object (smaller of size1-2) */
   struct objrec obj;   /* SAC Database Record */
   BFILE        *odfp,  /* pointer to object data file */
                *outfp; /* pointer to output file */
   char         tmpbuff[MAXLINE+1], /* temp storage for output messages */
                *infilename,        /* name of the input file */
                *outfilename;       /* name of the output file */

   /* set defaults */
   tele = 0.0;
   sb0 = 0.0;
   prune = FALSE;
   dump = FALSE;
   format = FALSE;
   infilename = (char *) NULL;
   outfilename = (char *) NULL;

   /* process command line args */
   i = 0;
   while (++i < argc) {
      if ((argv[i][0] == '-') || (argv[i][0] == '/')) {
         if (i >= argc) {
            fprintf (stderr, "Error: Missing option argument\n");
            usage();
            exit(1);
         }
         switch (argv[i][1]) {
            case 'b':
            case 'B':
               sb0 = atof(argv[++i]);
               break;
            case 'a':
            case 'A':
               tele = atof(argv[++i]);
               break;
            case 'i':
            case 'I':
               infilename = argv[++i];
               break;
            case 'o':
            case 'O':
               outfilename = argv[++i];
               break;
            case 'd':
            case 'D':
               if (format) {
                  fprintf (stderr, "Error: dump and format are mutually exclusive options\n");
                  usage();
                  exit(1);
               }
               dump++;
               break;
            case 'f':
            case 'F':
               if (dump) {
                  fprintf (stderr, "Error: dump and format are mutually exclusive options\n");
                  usage();
                  exit(1);
               }
               format++;
               break;
            case 'p':
            case 'P':
               prune++;
               break;
            default:
               fprintf (stderr, "Error: Invalid option: %s\n", argv[i]);
               usage();
               exit (1);
         } 
      } else {
         /* argv[i][0] is not an option specifier */
         fprintf (stderr, "Error: Invalid command line\n");
         usage();
         exit (1);
      }
   }

   getinfile(infilename,&odfp);
   getoutfile(outfilename,&outfp);
   getenvdat(&tele,&sb0);
   calcenvdat(tele,&tmag,&ipmin);

   /* start processing objects, looping until no more input */
   while (ret=getobjdat(odfp,&obj)) {
      if (ret == BADDAT)
         continue;
      if (!convobjdat(&obj,&xmag,&size1,&size2)) {
         xmag = 99.9;
         size1 = 0.0;
         size2 = 0.0;
      }
      /* check if total magnitude is bright enough to be seen */
      if (tmag < xmag) {
         odm=0;
         c1=0;
         xc=0;
      } else {
         calcobjdat(xmag,size1,size2,sb0,&sb,&c1,&xminsz,&sbs);
         calcodm(sb0,xminsz,tele,ipmin,sbs,&mb,&sbp,&sbm,&odm,&xc);
      }
      if (dump && prune) {
         if ((xmag==99.9) || (xmag==79.9) || (xc<c1)) {
            fulldump(outfp,&obj,odm,c1,xc);
         }
      } else if (format && prune) {
         if ((xmag==99.9) || (xmag==79.9) || (xc<c1)) {
            formatout(outfp,&obj,odm,c1,xc,tele,ipmin,tmag,sb0);
         }
      } else if (dump) {
         fulldump(outfp,&obj,odm,c1,xc);
      } else if (prune) {
         if ((xmag==99.9) || (xmag==79.9) || (xc<c1)) {
            sprintf (tmpbuff,"%s\n",obj.rawdat);
            bputarr (outfp, strlen(tmpbuff), tmpbuff);
         }
      } else
         formatout(outfp,&obj,odm,c1,xc,tele,ipmin,tmag,sb0);
   }
   (void) bclosein(&odfp);
   (void) bcloseout(&outfp);
   exit (0);
}


/*
   Function: usage
   Purpose:  Print out a usage message for the program.
*/

#ifdef __STDC__
void usage(void)
#else
void usage()
#endif
{
   fprintf (stderr, "Usage: sacodm [-b bsb] [-a aperture] [-d | -f] [-p] [-i infile] [-o outfile]\n");
}



/*
   Function: getinfile
   Purpose:  Prompt for the name of a file (if not already set), then open the
             file for reading.

        infilename = pointer to the name of the input file
        fpp = pointer to opened file
*/

#ifdef __STDC__
void getinfile(char *infilename, BFILE **fpp)
#else
void getinfile(infilename,fpp)
   char *infilename;
   BFILE **fpp;
#endif
{
   char name[MAXLINE+1];        /* answer to file name prompt */

   if (infilename == (char *) NULL) {
      printf ("Enter the name of the input file: ");
      if (getline(name,MAXLINE) < 1) {
         fprintf (stderr, "Missing or invalid file name\n");
         exit (1);
      }
      infilename = name;
   }
   switch (bopenin(infilename,fpp)) {
      case NOBUFF:
         fprintf (stderr, "Can't allocate buffer for input file.\n");
         exit(1);
         break;
      case BADOPEN:
         fprintf (stderr, "Can't open %s for reading.\n",infilename);
         exit (1);
         break;
      default:
         break;
   }
}


/*
   Function: getoutfile
   Purpose:  Prompt for the name of a file (if not already set), then open the
             file for writing.
             If a file exists by the same name, ask if it should be wiped.

        outfilename = pointer to the name of the output file
        fpp = pointer to opened file
*/

#ifdef __STDC__
void getoutfile(char *outfilename,BFILE **fpp)
#else
void getoutfile(outfilename,fpp)
   char *outfilename;
   BFILE **fpp;
#endif
{
   char name[MAXLINE+1],        /* answer to file name prompt */
        answer[MAXLINE+1];      /* reply to question about overwrite */
   struct stat statbuf;         /* storage for info from stat */

   if (outfilename == (char *) NULL) {
      printf ("Enter the name of the output file: ");
      if (getline(name,MAXLINE) < 1) {
         fprintf (stderr, "Missing or invalid file name\n");
         exit (1);
      }
      outfilename = name;
   }
   if (stat(outfilename,&statbuf) == 0) {
      fprintf (stderr, "File \"%s\" already exists, overwrite? (y/n) ",outfilename);
      getline(answer,MAXLINE);
      if (answer[0] != 'y') {
         fprintf (stderr, "Aborting.\n");
         exit(1);
      }
   }
   switch (bopenout(outfilename,fpp)) {
      case NOBUFF:
         fprintf (stderr, "Can't allocate buffer for output file.\n");
         exit(1);
         break;
      case BADOPEN:
         fprintf (stderr, "Can't open %s for writing.\n",outfilename);
         exit (1);
         break;
      default:
         break;
   }
}


/* Function: getline
   Purpose:  Reads a line from stdin

   Pre: buff is a character array used to store the characters plus a NULL.
        There must be room in buff for max chars plus a NULL.

   Post: Chars are read from stdin until a '\n' is read or EOF is reached.
         At most max chars are stored in buff.
         buff is terminated with a NULL.
         The trailing '\n' is discarded.
         Chars after the first max chars are discarded.
         Get_Line returns the number of chars read or EOF.
*/

#ifdef __STDC__
int getline (char buff[], int max)
#else
int getline (buff, max)
   char buff[];
   int max;
#endif
{
   register int c,
                i = 0;

   while (((c = getchar()) != EOF) && (c != '\n'))
      if (i < max)
         buff[i++] = (char) c;
      else if (i == max)
         buff[i++] = (char) NULL;
      else
         i++;
   if (i <= max)
      buff[i] = (char) NULL;

   if ((i == 0) && (c == EOF))
      return EOF;
   else
      return i;
}



/*
   Function: getenvdat
   Purpose: Get the environmental data in interactive mode.
        tele = telescope aperture in inches
        sb0  = background surface brightness
*/

#ifdef __STDC__
void getenvdat(double *tele, double *sb0)
#else
void getenvdat(tele,sb0)
   double *tele, *sb0;
#endif
{
   char input[MAXLINE+1];
   int i;

   if (*tele == 0.0) {
      printf ("Enter the telescope aperture in inches: ");
      getline (input,MAXLINE);
      for (i=0;isspace(input[i]);i++)
         ;
      *tele = atof(&(input[i]));
   }
   if (*tele <= .01) {
      fprintf (stderr, "Invalid aperture\n");
      exit (1);
   }

   if (*sb0 == 0.0) {
      printf ("Enter the background surface brightness (24.25 = country -> 19 = city): ");
   getline (input,MAXLINE);
      for (i=0;isspace(input[i]);i++)
         ;
      *sb0 = atof(&(input[i]));
   }
   if ((*sb0 < 10) || (*sb0 > 27)) {
      fprintf (stderr, "Invalid surface brightness\n");
      exit (1);
   }
}


/*
   Function: getfield
   Purpose: Extract the quoted text from the beginning of a character string.
            The string should beging either with a double quote, or with a
            comma followed by the double quote.  The comma and/or double quote
            are skipped, and the characters up to (but not including) the next
            double quote are copied to the output string.

        string is a pointer to the input string
        max is the maximum number of characters to copy to field
        field is a pointer to the output string

   Returns a pointer to the character following the terminating double quote.
   On error, a NULL pointer is returned.
*/

#ifdef __STDC__
char *getfield(char *string, int max, char *field)
#else
char *getfield(string,max,field)
   char *string, *field;
   int max;
#endif
{
   char *s,     /* pointer into string */
        *f,     /* pointer into field */
        *first; /* first character in string that gets copied to field */

   s = string;
   f = field;
   if (*s == ',')
      s++;
   if (*(s++) != '"')
      return ((char *) NULL);
   first = s;
   /* copy characters up to and including the terminating double quote */
   while (((*f = *(s++)) != '"') && (s <= first + max))
      f++;
   if (*f == '"') {
      *f = '\0';
      return (s);
   } else {
      *field = '\0';
      return ((char *) NULL);
   }
}


/*
   Function: getobjdat
   Purpose: Get data about the object.  The data is expected to be in the
            format of version 6.0 of the SAC Deep Space database.
            Blank lines, and those starting with '#' are ignored.

        fp is a pointer to the open input file
        obj is a pointer to an objrec record.

   Returns TRUE if data is found, FALSE if not.
*/

#ifdef __STDC__
int getobjdat(BFILE *fp, struct objrec *obj)
#else
int getobjdat(fp,obj)
   BFILE *fp;
   struct objrec *obj;
#endif
{
   int c;
   char *next;

   while ((c = bgetstr (fp,MAXLINE,obj->rawdat)) > 0) {
      if ((obj->rawdat[0] == '#') || (obj->rawdat[0] == '\0'))
         continue;
      else
         break;
   }
   if (c == 0)
      return (FALSE);

   next = obj->rawdat;
   if ((next = getfield(next,OBJECTSZ,obj->object)) == (char *) NULL) {
      fprintf (stderr,"Missing or invalid OBJECT field.\n");
      fprintf (stderr,"Offending record:\n");
      fprintf (stderr,"%s\n",obj->rawdat);
      return (BADDAT);
   }
   if ((next = getfield(next,OTHERSZ,obj->other)) == (char *) NULL) {
      fprintf (stderr,"Missing or invalid OTHER field.\n");
      fprintf (stderr,"Offending record:\n");
      fprintf (stderr,"%s\n",obj->rawdat);
      return (BADDAT);
   }
   if ((next = getfield(next,TYPESZ,obj->type)) == (char *) NULL) {
      fprintf (stderr,"Missing or invalid TYPE field.\n");
      fprintf (stderr,"Offending record:\n");
      fprintf (stderr,"%s\n",obj->rawdat);
      return (BADDAT);
   }
   if ((next = getfield(next,CONSZ,obj->con)) == (char *) NULL) {
      fprintf (stderr,"Missing or invalid CON field.\n");
      fprintf (stderr,"Offending record:\n");
      fprintf (stderr,"%s\n",obj->rawdat);
      return (BADDAT);
   }
   if ((next = getfield(next,RASZ,obj->ra)) == (char *) NULL) {
      fprintf (stderr,"Missing or invalid RA field.\n");
      fprintf (stderr,"Offending record:\n");
      fprintf (stderr,"%s\n",obj->rawdat);
      return (BADDAT);
   }
   if ((next = getfield(next,DECSZ,obj->dec)) == (char *) NULL) {
      fprintf (stderr,"Missing or invalid DEC field.\n");
      fprintf (stderr,"Offending record:\n");
      fprintf (stderr,"%s\n",obj->rawdat);
      return (BADDAT);
   }
   if ((next = getfield(next,MAGSZ,obj->mag)) == (char *) NULL) {
      fprintf (stderr,"Missing or invalid mag field.\n");
      fprintf (stderr,"Offending record:\n");
      fprintf (stderr,"%s\n",obj->rawdat);
      return (BADDAT);
   }
   if ((next = getfield(next,SUBRSZ,obj->subr)) == (char *) NULL) {
      fprintf (stderr,"Missing or invalid SUBR field.\n");
      fprintf (stderr,"Offending record:\n");
      fprintf (stderr,"%s\n",obj->rawdat);
      return (BADDAT);
   }
   if ((next = getfield(next,U2000SZ,obj->u2000)) == (char *) NULL) {
      fprintf (stderr,"Missing or invalid U2000 field.\n");
      fprintf (stderr,"Offending record:\n");
      fprintf (stderr,"%s\n",obj->rawdat);
      return (BADDAT);
   }
   if ((next = getfield(next,S2000SZ,obj->s2000)) == (char *) NULL) {
      fprintf (stderr,"Missing or invalid S2000 field.\n");
      fprintf (stderr,"Offending record:\n");
      fprintf (stderr,"%s\n",obj->rawdat);
      return (BADDAT);
   }
   if ((next = getfield(next,DESCRSZ,obj->descr)) == (char *) NULL) {
      fprintf (stderr,"Missing or invalid DESCR field.\n");
      fprintf (stderr,"Offending record:\n");
      fprintf (stderr,"%s\n",obj->rawdat);
      return (BADDAT);
   }
   if ((next = getfield(next,SIZESZ,obj->size)) == (char *) NULL) {
      fprintf (stderr,"Missing or invalid SIZE field.\n");
      fprintf (stderr,"Offending record:\n");
      fprintf (stderr,"%s\n",obj->rawdat);
      return (BADDAT);
   }
   if ((next = getfield(next,CLASSSZ,obj->class)) == (char *) NULL) {
      fprintf (stderr,"Missing or invalid CLASS field.\n");
      fprintf (stderr,"Offending record:\n");
      fprintf (stderr,"%s\n",obj->rawdat);
      return (BADDAT);
   }
   if ((next = getfield(next,NSTSSZ,obj->nsts)) == (char *) NULL) {
      fprintf (stderr,"Missing or invalid NSTS field.\n");
      fprintf (stderr,"Offending record:\n");
      fprintf (stderr,"%s\n",obj->rawdat);
      return (BADDAT);
   }
   if ((next = getfield(next,BRSTRSZ,obj->brstr)) == (char *) NULL) {
      fprintf (stderr,"Missing or invalid BRSTR field.\n");
      fprintf (stderr,"Offending record:\n");
      fprintf (stderr,"%s\n",obj->rawdat);
      return (BADDAT);
   }
   if ((next = getfield(next,NOTESZ,obj->note)) == (char *) NULL) {
      fprintf (stderr,"Missing or invalid NOTE field.\n");
      fprintf (stderr,"Offending record:\n");
      fprintf (stderr,"%s\n",obj->rawdat);
      return (BADDAT);
   }
   return (TRUE);
}


/*
   Function: convobjdat
   Purpose: convert the text mode data in obj to numeric data for
            calculations.
            The obj->mag field should contain a simple decimal number in text
            mode.
            The obj->size field should contain:
                a decimal number in text mode
                one or two single quotes
                optionally followed by:
                        an x or X
                        another decimal number in text mode
                        one or two single quotes
            If a decimal number in obj->size is followed by one single quote,
            the number is arc-minutes.  If it is followed by two single quotes,
            it is in arc-seconds.
            In either case, the number is converted to arc-minutes.
            If only the first number is given (ie no xNNN), we assume that this
            is a diameter, and assign the value to both size and size2.

   Returnes TRUE on success, FALSE on failure.

        obj = the text SAC database record
        xmag = pointer to the objects visual magnitude
        size1 = pointer to first dimension (in arc minutes)
        size2 = pointer to second dimension (in arc minutes)
*/

#ifdef __STDC__
int convobjdat(struct objrec *obj, double *xmag, double *size1, double *size2)
#else
int convobjdat(obj,xmag,size1,size2)
   struct objrec *obj;
   double *xmag, *size1, *size2;
#endif
{
   char *curr,  /* pointer to the "current" sub-field in obj-size */
        *next;  /* pointer to "next" sub-field in obj->size */

   if ((*xmag = atof(obj->mag)) == 0) {
      return(FALSE);
   }
   curr = obj->size;
   *size1 = strtod(curr,&next);
   if ((next == curr) || (*next != '\'')) {
      return(FALSE);
   }
   if (*(++next) == '\'') {
      /* number is arc-seconds, convert to arc-minutes */
      *size1 /= 60;
      next++;
   }
   if ((*next != 'x') && (*next != 'X')) {
      if (*next == '\0') {
         /* only one dimension, must be diameter */
         *size2 = *size1;
         return(TRUE);
      } else {
         return(FALSE);
      }
   }
   curr = next + 1;
   *size2 = strtod(curr,&next);
   if ((next == curr) || (*next != '\'')) {
      return(FALSE);
   }
   if (*(++next) == '\'') {
      /* number is arc-seconds, convert to arc-minutes */
      *size2 /= 60;
      next++;
   }
   if (*next != '\0') {
      return(FALSE);
   }
   return(TRUE);
}


/*
   Function: fulldump
   Purpose:  Print out the raw input record for an object, followed by
             the log contrast, contrast threshold, optimum detection
             magnification, and a field that is 'Y' if the object is known to
             be detectable, and ' ' otherwise.  The additional information
             is printed in the same format as the original SAC database (comma
             seperated, enclosed in quotes).
*/

#ifdef __STDC__
void fulldump(BFILE *outfp, struct objrec *obj, int odm, double c1, double xc)
#else
void fulldump(outfp,obj,odm,c1,xc)
   BFILE *outfp;
   struct objrec *obj;
   int odm;
   double c1,xc;
#endif
{
   char tmpbuff[MAXLINE+1];

   sprintf (tmpbuff,"%s,\"%4.1f\",\"%4.1f\",\"%4d\",\"%c\"\n",obj->rawdat,c1,xc,odm,(xc<c1) ? 'Y' : ' ');
   bputarr (outfp, strlen(tmpbuff), tmpbuff);
}



/*
   Function: formatout
   Purpose: print out the results of the calculations for the object
            Prints a header the first time it is called, then prints all data
            for a single object on one line.  Subsequent calls just print the
            data for a new object.
*/

#ifdef __STDC__
void formatout(BFILE *outfp, struct objrec *obj, int odm, double c1, double xc,
               double tele, int ipmin, double tmag, double sb0)
#else
void formatout(outfp,obj,odm,c1,xc,tele,ipmin,tmag,sb0)
   BFILE *outfp;
   struct objrec *obj;
   int odm, ipmin;
   double c1,xc,tele,tmag,sb0;
#endif
{
   static int firsttime = TRUE;
   char tmpbuff[MAXLINE+1];

   if (firsttime) {
      /* print the header only the first time around... */
      sprintf (tmpbuff,"\nTelescope aperture: %5.1f inches\n",tele);
      bputarr (outfp, strlen(tmpbuff), tmpbuff);
      sprintf (tmpbuff,"Minimum usable magnification: %dx\n",ipmin);
      bputarr (outfp, strlen(tmpbuff), tmpbuff);
      sprintf (tmpbuff,"Faintest star: %5.1f\n",tmag);
      bputarr (outfp, strlen(tmpbuff), tmpbuff);
      sprintf (tmpbuff,"\nBackground Surface Brightness: %5.2f\n",sb0);
      bputarr (outfp, strlen(tmpbuff), tmpbuff);
      sprintf (tmpbuff,"\n                                                             Log  Con\n");
      bputarr (outfp,strlen(tmpbuff),tmpbuff);
      sprintf (tmpbuff,"    Object    Type   Con    R.A.     Dec   Mag      Size     Con  Thrs ODM  D\n");
      bputarr (outfp,strlen(tmpbuff),tmpbuff);
      sprintf (tmpbuff,"-------------------------------------------------------------------------------\n");
      bputarr (outfp,strlen(tmpbuff),tmpbuff);
      firsttime = FALSE;
   }
   sprintf (tmpbuff,"%-13s %4s  %3s  %7s  %6s  %4s %11s %4.1f %4.1f %4d  %c\n"
           ,obj->object,obj->type,obj->con,obj->ra,obj->dec,obj->mag,obj->size,c1,xc,odm,(xc<c1) ? 'Y' : ' ');
   bputarr (outfp,strlen(tmpbuff),tmpbuff);
}
