/*
   Program: teleodm.c
   Version: 1.1 Beta
   Original Author: Roger N. Clark
   Translated to C by: Dave Lutz
   Reference: "Visual Astronomy of the Deep Sky", Clark R., 1990 Sky Publishing
              Corporation, ISBN  0-933-346-54-9, and Cambridge University
              Press, ISBN 0-521-36155-9
   Bug reports to: lutz@scp.rochester.edu or lutz@bcs.rochester.edu

   Copyright: The original work is Copyright Roger N. Clark 1990, and is used
              by permission.
              This version is Copyright David L. Lutz 1995.

   Disclaimer:  This software is distributed free of charge.  As such, it
                comes with ABSOLUTELY NO WARRANTY.  The user of the software
                assumes ALL RISKS associated with its use.

   Purpose:

        When provided with the aperture of a telesope, the background surface
        brightness of the sky, and the size and magnitude of an object, the
        program determines the Optimum Detection Magnification for the object.

   Modifications from the original:

        07/12/94 - query the user for the background surface brightness
        07/12/94 - print out telescope info before asking for object info
        07/12/94 - add loop to ask for multiple objects without quiting
        07/13/94 - modularize
        07/13/94 - add batch mode, which doesn't print prompts.  Data format
                   is described below.
        07/14/94 - batch mode output is in table form

  Batch Mode File format:

        - Blank lines, and those beginning with '#' are ignored in the input.
        - The first data line in the file should contain environment data. This
          should be aperture (in inches) followed by background surface
          brightness, separated by spaces.
        - The remainder of the environment line is ignored, and can be used for
          a comment.
        - Second and subsequent lines contain object data.  This should be
          total visual magnitude, followed by the dimensions of the object in
          arc-minutes, all separated by spaces.
        - the remainder of the object line is ignored, and can be used for a
          comment.

        Example:

        8 24.25      (8" scope in dark country sky)
        11.4  1  5   (NGC 134)
         8.8 60 60   (Sculptor System)
*/

#include <stdio.h>
#include <math.h>
#include <string.h>
#include "odm.h"

#define TRUE 1
#define FALSE 0

#define MAXLINE 1024

#ifdef __STDC__
int getline(char buff[], int max);
void ienvdat(double *tele, double *sb0);
void benvdat(double *tele, double *sb0);
int iobjdat(double *xmag, double *size1, double *size2);
int bobjdat(double *xmag, double *size1, double *size2);
void iresults(double size1, double size2, double sb, double mb, double sbp,
              double sbm, int odm, double c1, double xc);
void bresults(double size1, double size2, double sb, double mb, double sbp,
              double sbm, int odm, double c1, double xc);
#else
int getline();
void ienvdat();
void benvdat();
int iobjdat();
int bobjdat();
void iresults();
void bresults();
#endif

#ifdef __STDC__
int main (int argc, char **argv)
#else
int main (argc, argv)
   int argc;
   char **argv;
#endif
{

   int          odm,    /* optimum detection magnification */
                ipmin,  /* minimum useful magnification for telescope */
                errflg, /* is command line ok? */
                batch;  /* use batch mode? */
   double       tele,   /* telescope's aperture in inches */
                mb,     /* surface brightness reduction at power */
                sbm,    /* surface brightness combined with sb reduction */
                tmag,   /* telescope's limiting magnitude */
                xmag,   /* total magnitude of object */
                sb0,    /* background surface brightness */
                sb,     /* computed surface brightness of object */
                sbs,    /* sb0 combined with sb */
                sbp,    /* surface brightness of object at odm */
                c1,     /* computed contrast index */
                xc,     /* contrast of object at odm */
                size1,  /* one dimension of the object (in arc-minutes) */
                size2,  /* one dimension of the object (in arc-minutes) */
                xminsz; /* minimum size of object (smaller of size1-2) */

   void (*envdatread)(), /* pointer to function that reads in env data */
        (*results)();    /* pointer to function that prints results */
   int (*objdatread)();  /* pointer to function that reads in object data */

   errflg = FALSE;
   batch = FALSE;
   if (argc == 2)
      if ((argv[1][0] == '-' || argv[1][0] == '/') &&
          (argv[1][1] == 'b' || argv[1][1] == 'B'))
         batch = TRUE;
      else
         errflg = TRUE;
   else if (argc != 1)
      errflg = TRUE;
   if (errflg) {
      fprintf (stderr,"Usage: teleodm [-b]\n");
      exit (1);
   }

   printf ("***** Program Teleodm: Telescope Optimum Detection Magnification *****\n");

   if (batch) {
      envdatread = benvdat;
      objdatread = bobjdat;
      results = bresults;
   } else {
      envdatread = ienvdat;
      objdatread = iobjdat;
      results = iresults;
   }

   envdatread(&tele,&sb0);
   calcenvdat(tele,&tmag,&ipmin);

   printf ("\nTelescope aperture: %7.1f inches\n",tele);
   printf ("Minimum usable magnification: %5dx\n",ipmin);
   printf ("Faintest star: %6.1f\n\n",tmag);

   /* start processing objects, looping until no more input */
   while ((*objdatread)(&xmag,&size1,&size2)) {

      /* check if total magnitude is bright enough to be seen */
      if (tmag < xmag) {
         printf ("Magnitude %6.1f is too faint to be seen in a",xmag);
         printf ("  %6.1f inch telescope.\n",tele);
         continue;
      }

      calcobjdat(xmag,size1,size2,sb0,&sb,&c1,&xminsz,&sbs);
      calcodm(sb0,xminsz,tele,ipmin,sbs,&mb,&sbp,&sbm,&odm,&xc);
      results(size1,size2,sb,mb,sbp,sbm,odm,c1,xc);
   }
   exit (0);
}


/* Function: getline
   Purpose:  Reads a line from stdin

   Pre: buff is a character array used to store the characters plus a NULL.
        There must be room in buff for max chars plus a NULL.

   Post: Chars are read from stdin until a '\n' is read or EOF is reached.
         At most max chars are stored in buff.
         buff is terminated with a NULL.
         The trailing '\n' is discarded.
         Chars after the first max chars are discarded.
         Get_Line returns the number of chars read or EOF.
*/

#ifdef __STDC__
int getline (char buff[], int max)
#else
int getline (buff, max)
   char buff[];
   int max;
#endif
{
   register int c,
                i = 0;

   while (((c = getchar()) != EOF) && (c != '\n'))
      if (i < max)
         buff[i++] = (char) c;
      else if (i == max)
         buff[i++] = (char) NULL;
      else
         i++;
   if (i <= max)
      buff[i] = (char) NULL;

   if ((i == 0) && (c == EOF))
      return EOF;
   else
      return i;
}


/*
   Function: ienvdat
   Purpose: Get the environmental data in interactive mode.
        tele = telescope aperture in inches
        sb0  = background surface brightness
*/

#ifdef __STDC__
void ienvdat(double *tele, double *sb0)
#else
void ienvdat(tele,sb0)
   double *tele, *sb0;
#endif
{
   char input[MAXLINE+1];
   int i;

   printf ("Enter the telescope aperture in inches: ");
   getline (input,MAXLINE);
   for (i=0;isspace(input[i]);i++)
      ;
   *tele = atof(&(input[i]));
   if (*tele <= .01) {
      fprintf (stderr, "Invalid aperture\n");
      exit (1);
   }

   printf ("Enter the background surface brightness (24.25 = country -> 19 = city): ");
   getline (input,MAXLINE);
   for (i=0;isspace(input[i]);i++)
      ;
   *sb0 = atof(&(input[i]));
   if ((*sb0 < 10) || (*sb0 > 27)) {
      fprintf (stderr, "Invalid surface brightness\n");
      exit (1);
   }
}


/*
   Function: benvdat
   Purpose: Get the environmental data in batch mode.  No prompts are printed,
            and the data is expected to be on one line, separated by spaces.
            Blank lines, and those starting with '#' are ignored.
        tele = telescope aperture in inches
        sb0  = background surface brightness
*/

#ifdef __STDC__
void benvdat(double *tele, double *sb0)
#else
void benvdat(tele,sb0)
   double *tele, *sb0;
#endif
{
   char input[MAXLINE+1];
   int i;

   while (getline (input,MAXLINE) != EOF) {
      for (i=0;isspace(input[i]);i++)
         ;
      if ((input[i] == '#') || (input[i] == '\0'))
         continue;
      else
         break;
   }
   *tele = atof(&(input[i]));
   if (*tele <= .01) {
      fprintf (stderr, "Invalid aperture\n");
      exit (1);
   }

   for (;(input[i] != '\0') && !isspace(input[i]);i++)
      ;
   if (input[i] == '\0') {
      fprintf (stderr, "Missing background surface brightness\n");
      exit (1);
   }
   for (;isspace(input[i]);i++)
      ;
   *sb0 = atof(&(input[i]));
   if ((*sb0 < 10) || (*sb0 > 27)) {
      fprintf (stderr, "Invalid background surface brightness\n");
      exit (1);
   }
}


/*
   Function: iobjdat
   Purpose: Get object data in interactive mode.
        xmag = total visual magnitude of object
        size1 = one dimension of the object (in arc-minutes)
        size2 = the other dimension of the object (in arc-minutes)
   Returns TRUE if data is found, FALSE if not.
*/

#ifdef __STDC__
int iobjdat(double *xmag, double *size1, double *size2)
#else
int iobjdat(xmag,size1,size2)
   double *xmag, *size1, *size2;
#endif
{
   char input[MAXLINE+1];
   int i;

   printf ("\nEnter the total magnitude of an object (return to exit): ");
   if (getline (input,MAXLINE) < 1)
      return (FALSE);
   for (i=0;isspace(input[i]);i++)
      ;
   *xmag = atof(&(input[i]));
   if ((*xmag < .01) || (*xmag > 25)) {
      fprintf (stderr, "Invalid magnitude: %f\n",*xmag);
      exit (1);
   }

   printf ("Enter the minimum size of the object in arc-minutes: ");
   getline (input,MAXLINE);
   for (i=0;isspace(input[i]);i++)
      ;
   *size1 = atof(&(input[i]));
   if (*size1 <= 0.0) {
      fprintf (stderr, "Invalid minimum size: %f\n",*size1);
      exit (1);
   }

   printf ("Enter the maximum size of the object in arc-minutes: ");
   getline (input,MAXLINE);
   for (i=0;isspace(input[i]);i++)
      ;
   *size2 = atof(&(input[i]));
   if (*size2 <= 0.0) {
      fprintf (stderr, "Invalid maximum size: %f\n",size2);
      exit (1);
   }
   return (TRUE);
}


/*
   Function: bobjdat
   Purpose: Get data about the object in batch mode.  No prompts are printed,
            and the data is expected to be on one line, separated by spaces.
            Blank lines, and those starting with '#' are ignored.
        xmag = total visual magnitude of object
        size1 = one dimension of the object (in arc-minutes)
        size2 = the other dimension of the object (in arc-minutes)
   Returns TRUE if data is found, FALSE if not.
*/

#ifdef __STDC__
int bobjdat(double *xmag, double *size1, double *size2)
#else
int bobjdat(xmag,size1,size2)
   double *xmag, *size1, *size2;
#endif
{
   char input[MAXLINE+1];
   int i,c;


   while ((c = getline (input,MAXLINE)) != EOF) {
      for (i=0;isspace(input[i]);i++)
         ;
      if ((input[i] == '#') || (input[i] == '\0'))
         continue;
      else
         break;
   }
   if (c == EOF)
      return (FALSE);
   *xmag = atof(&(input[i]));
   if ((*xmag < .01) || (*xmag > 25)) {
      fprintf (stderr, "Invalid magnitude: %f\n",*xmag);
      exit (1);
   }

   for (;(input[i] != '\0') && !isspace(input[i]);i++)
      ;
   if (input[i] == '\0') {
      fprintf (stderr, "Missing minimum size\n");
      exit (1);
   }
   for (;isspace(input[i]);i++)
      ;
   *size1 = atof(&(input[i]));
   if (*size1 <= 0.0) {
      fprintf (stderr, "Invalid minimum size: %f\n",*size1);
      exit (1);
   }

   for (;(input[i] != '\0') && !isspace(input[i]);i++)
      ;
   if (input[i] == '\0') {
      fprintf (stderr, "Missing maximum size\n");
      exit (1);
   }
   for (;isspace(input[i]);i++)
      ;
   *size2 = atof(&(input[i]));
   if (*size2 <= 0.0) {
      fprintf (stderr, "Invalid maximum size: %f\n",size2);
      exit (1);
   }
   return (TRUE);
}


/*
   Function: iresults
   Purpose: print out the results of the calculations for the object
            Used in interactive mode.
*/

#ifdef __STDC__
void iresults(double size1, double size2, double sb, double mb, double sbp,
              double sbm, int odm, double c1, double xc)
#else
void iresults(size1,size2,sb,mb,sbp,sbm,odm,c1,xc)
   double size1,size2,sb,mb,sbp,sbm,c1,xc;
   int odm;
#endif
{
      printf ("\n-----------------------------------------------\n");
      printf ("Object size: %8.2f by %8.2f arc minutes\n",size1,size2);
      printf ("surface brightness (magnitude / sq. arc-sec):\n");
      printf ("\tobject with no telescope:       %7.1f\n",sb);
      printf ("\treduction due to odm:           %7.1f\n",mb);
      printf ("\tobject in telescope at odm:     %7.1f\n",sbp);
      printf ("\tbackground in telescope at odm: %7.1f\n\n",sbm);
      printf ("ODM: %dx\n\n",odm);
      printf ("Log object contrast:    %5.1f\n",c1);
      printf ("Log threshold contrast: %5.1f\n",xc);
      if (c1 < xc) {
         printf ("The CONTRAST IS TOO LOW TO DETECT the object.\n");
      } else {
         printf ("The object is detectable.\n");
      }
      printf ("-----------------------------------------------\n");
}


/*
   Function: bresults
   Purpose: print out the results of the calculations for the object
            Used in batch mode.  Prints a header the first time it is called,
            then prints all data for a single object on one line.  Subsequent
            calls just print the data for a new object.
*/

#ifdef __STDC__
void bresults(double size1, double size2, double sb, double mb, double sbp,
              double sbm, int odm, double c1, double xc)
#else
void bresults(size1,size2,sb,mb,sbp,sbm,odm,c1,xc)
   double size1,size2,sb,mb,sbp,sbm,c1,xc;
   int odm;
#endif
{
   static int firsttime = TRUE;

   if (firsttime) {
      /* print the header only the first time around... */
      printf ("                     reduct  obj     bg\n");
      printf ("                       at    at      at\n");
      printf ("    size      sb      odm    odm     odm    ODM   loc  ltc  detectable?\n");
      printf ("-------------------------------------------------------------------------------\n");
      firsttime = FALSE;
   }
   printf ("%5.1fx%-5.1f %5.1f %7.1f %7.1f %7.1f %5d  %4.1f %4.1f      %c\n",
           size1,size2,sb,mb,sbp,sbm,odm,c1,xc,(c1<xc) ? ' ' : 'Y');
}
